(* ****************************************************************************
 *                            ImageTrimmer_Main
 *                  Copyright (C) 2004-2007 Fumio Kawamata
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
 *  ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 *  Public License in the directory "gnu" for more details.
 ******************************************************************************
 *)
unit ImageTrimmer_Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ComCtrls, Menus, jpeg, StdCtrls, IniFiles, ImgList,
  Buttons, Mask;

type

  TFormMain = class(TForm)
    BitBtnDown:                                   TBitBtn;
    BitBtnExit:                                   TBitBtn;
    BitBtnOpen:                                   TBitBtn;
    BitBtnSaveAs:                                 TBitBtn;
    BitBtnUp:                                     TBitBtn;
    ComboBoxRatio:                                TComboBox;
    FontDialog:                                   TFontDialog;
    GroupBoxImageSize:                            TGroupBox;
    ImageList:                                    TImageList;
    ImageMaster:                                  TImage;
    ImageSave:                                    TImage;
    ImageView:                                    TImage;
    MainMenu:                                     TMainMenu;
    MainMenuDisplay:                              TMenuItem;
    MainMenuDisplayOriginalImageSize:             TMenuItem;
    MainMenuDisplayTrimminbBoxAtHomePotision:     TMenuItem;
    MainMenuEdit:                                 TMenuItem;
    MainMenuEditJPEGQuality:                      TMenuItem;
    MainMenuEditSetTrimmingSize:                  TMenuItem;
    MainMenuEditTrimmingSize:                     TMenuItem;
    MainMenuFile:                                 TMenuItem;
    MainMenuFileExit:                             TMenuItem;
    MainMenuFileLoad:                             TMenuItem;
    MainMenuFileSaveAs:                           TMenuItem;
    MainMenuHelp:                                 TMenuItem;
    MainMenuHelpAbout:                            TMenuItem;
    MainMenuImageSize100:                         TMenuItem;
    MainMenuImageSize10:                          TMenuItem;
    MainMenuImageSize20:                          TMenuItem;
    MainMenuImageSize30:                          TMenuItem;
    MainMenuImageSize40:                          TMenuItem;
    MainMenuImageSize50:                          TMenuItem;
    MainMenuImageSize60:                          TMenuItem;
    MainMenuImageSize70:                          TMenuItem;
    MainMenuImageSize80:                          TMenuItem;
    MainMenuImageSize90:                          TMenuItem;
    MainMenuImageSize:                            TMenuItem;
    MainMenuImageSizeEnlarge:                     TMenuItem;
    MainMenuImageSizeJustify:                     TMenuItem;
    MainMenuImageSizeShrink:                      TMenuItem;
    MainMenuOption:                               TMenuItem;
    MainMenuOptionFont:                           TMenuItem;
    MainMenuOptionHomeDir:                        TMenuItem;
    MainMenuOptionINIFilename:                    TMenuItem;
    MainMenuSetJPEGQuality:                       TMenuItem;
    OpenDialog:                                   TOpenDialog;
    PanelBottom:                                  TPanel;
    PanelSide:                                    TPanel;
    SaveDialog:                                   TSaveDialog;
    ScrollBox:                                    TScrollBox;
    StatusBar:                                    TStatusBar;
    RadioGroupTrimmingBox:                        TRadioGroup;
    MainMenuOptionTrimmingBox:                    TMenuItem;
    MainMenuOptionTrimmingBoxFixed:               TMenuItem;
    MainMenuOptionTrimmingBoxAutoResize:          TMenuItem;
    MainMenuOptionAutoTracking:                   TMenuItem;
    procedure MainMenuHelpAboutClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MainMenuEditFontClick(Sender: TObject);
    procedure MainMenuFileExitClick(Sender: TObject);
    procedure MainMenuFileLoadClick(Sender: TObject);
    procedure ComboBoxRatioChange(Sender: TObject);
    procedure MainMenuOptionFontClick(Sender: TObject);
    procedure MainMenuEditSetTrimmingSizeClick(Sender: TObject);
    procedure PaintBoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintBoxMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintBoxMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure MainMenuDisplayTrimminbBoxAtHomePotisionClick(
      Sender: TObject);
    procedure MainMenuFileSaveAsClick(Sender: TObject);
    procedure MainMenuImageSizeEnlargeClick(Sender: TObject);
    procedure MainMenuImageSizeShrinkClick(Sender: TObject);
    procedure MainMenuImageSize100Click(Sender: TObject);
    procedure MainMenuImageSize90Click(Sender: TObject);
    procedure MainMenuImageSize80Click(Sender: TObject);
    procedure MainMenuImageSize70Click(Sender: TObject);
    procedure MainMenuImageSize60Click(Sender: TObject);
    procedure MainMenuImageSize50Click(Sender: TObject);
    procedure MainMenuImageSize40Click(Sender: TObject);
    procedure MainMenuImageSize30Click(Sender: TObject);
    procedure MainMenuImageSize20Click(Sender: TObject);
    procedure MainMenuImageSize10Click(Sender: TObject);
    procedure MainMenuImageSizeJustifyClick(Sender: TObject);
    procedure MainMenuSetJPEGQualityClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure MainMenuOptionTrimmingBoxFixedClick(Sender: TObject);
    procedure MainMenuOptionTrimmingBoxAutoResizeClick(Sender: TObject);
    procedure RadioGroupTrimmingBoxClick(Sender: TObject);
    procedure ScrollBoxMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure ScrollBoxMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure FormMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure FormMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
  private
    procedure EnableControls(Enabled: Boolean);
    procedure DisplayTrimmingBox;
    procedure DisplayTrimmingSize;
    procedure DisplayJPEGQuality;
    procedure ReadFormStatusFromIniFile(Form: TForm);
    procedure ReadOptionsFromFile(OptFileFullPath: string);
    procedure ResizeImage(Ratio: Integer);
    procedure WriteFormStatusToIniFile(Form: TForm);
    procedure WriteOptionsToFile(OptFileFullPath: string);
    procedure UnloadedWarning;
  public
  protected
    procedure CMDialogKey(var Message: TCMDialogKey);
    message CM_DIALOGKEY;
  end;

var
  FormMain: TFormMain;

implementation

{$R *.dfm}

uses VerInfo, AdjImgSize;

const
  CR = chr($0D)+chr($0A);
  IniFileName = 'ImageTrimmer.ini';

var
    HomeDir:               String;
    ImageRatio:            Single; (* 1 for full size *)
    IniFileFullPath:       String;
    JPEGQuality:           Integer;
    LastOpenDir:           String;
    LastSaveDir:           String;
    MarkerX:               Integer; (* Marker's center position in ImageMaster *)
    MarkerY:               Integer; (* Marker's center position in ImageMaster *)
    MouseX:                Integer;
    MouseY:                Integer;
    Msg:                   TMessage = (Msg: CM_PARENTFONTCHANGED);
    TrimmingWidth:         Integer;
    TrimmingHeight:        Integer;
    TrimmingTop:           Integer;
    TrimmingLeft:          Integer;
    TrimmingBoxIsSelected: Boolean;
    OpenedFilename:        String;
    OriginalCursor:        TCursor;


procedure TFormMain.UnloadedWarning;
begin
  MessageDlg('A file is not opened.  Open a file first.', mtWarning, [mbOK], 0);
end;

procedure TFormMain.EnableControls(Enabled: Boolean);
begin
  if Enabled=True then
  begin
    MainMenuFileSaveAs.Enabled:=True;
    MainMenuDisplay.Enabled:=True;
    MainMenuEdit.Enabled:=True;
    MainMenuImageSize.Enabled:=True;
    MainMenuOptionTrimmingBox.Enabled:=True;
    GroupBoxImageSize.Enabled:=True;
    RadioGroupTrimmingBox.Enabled:=True;
    BitBtnSaveAs.Enabled:=True;
  end
  else begin
    MainMenuFileSaveAs.Enabled:=False;
    MainMenuDisplay.Enabled:=False;
    MainMenuEdit.Enabled:=False;
    MainMenuImageSize.Enabled:=False;
    MainMenuOptionTrimmingBox.Enabled:=False;
    GroupBoxImageSize.Enabled:=False;
    RadioGroupTrimmingBox.Enabled:=False;
    BitBtnSaveAs.Enabled:=False;
  end;
end;

procedure TFormMain.CMDialogKey(var Message: TCMDialogKey);
var
  KeyState: TKeyBoardState;
procedure MoveTrimmingBox(X: Integer; Y: Integer);
var
  NewTrimmingLeft: Integer;
  NewTrimmingTop:  Integer;
  VirtualTrimmingWidth: Integer;
  VirtualTrimmingHeight: Integer;
begin
  if OpenedFilename='' then
    Exit;
  if (KeyState[VK_SHIFT] shr 7)=1 then
  begin
    X:=X*10;
    Y:=Y*10;
  end;
  NewTrimmingLeft:=TrimmingLeft+X;
  NewTrimmingTop:=TrimmingTop+Y;

  if MainMenuOptionTrimmingBoxFixed.Checked=True then
  begin
    VirtualTrimmingWidth:=TrimmingWidth;
    VirtualTrimmingHeight:=TrimmingHeight;
  end
  else begin
    VirtualTrimmingWidth:=Round(TrimmingWidth*ImageRatio);
    VirtualTrimmingHeight:=Round(TrimmingHeight*ImageRatio);
  end;

  if NewTrimmingLeft<0 then
    NewTrimmingLeft:=0;
  if NewTrimmingTop<0 then
    NewTrimmingTop:=0;
  if (NewTrimmingLeft+VirtualTrimmingWidth)>(ImageView.Width-1) then
      NewTrimmingLeft:=ImageView.Width-VirtualTrimmingWidth;
  if (NewTrimmingTop+VirtualTrimmingHeight)>(ImageView.Height-1) then
      NewTrimmingTop:=ImageView.Height-VirtualTrimmingHeight;

  DisplayTrimmingBox;
  TrimmingLeft:=NewTrimmingLeft;
  TrimmingTop:=NewTrimmingTop;
  DisplayTrimmingBox;
end;

begin
  GetKeyBoardState(KeyState);
  case Message.CharCode of
    VK_LEFT:  MoveTrimmingBox(-1,0);
    VK_RIGHT: MoveTrimmingBox(1,0);
    VK_UP:    MoveTrimmingBox(0,-1);
    VK_DOWN:  MoveTrimmingBox(0,1);
  else
    inherited;
    exit;
  end;
  Message.Result:=1;
end;

procedure TFormMain.MainMenuHelpAboutClick(Sender: TObject);
var
  Mess : String;
begin
  Mess:=VerInfo.GetVersionInfo('ProductName')+CR
       +VerInfo.GetVersionInfo('FileDescription')+CR
       +'Version '+VerInfo.GetVersionInfo('FileVersion')+CR
       +VerInfo.GetVersionInfo('LegalCopyright')+CR
       +'Freesoftware distributed under GNU GPL version 2'+CR
       +'Homepage: http://openlab.jp/fumio/';
  MessageDlg(Mess,mtInformation,[mbOK],0);
end;


procedure TFormMain.FormCreate(Sender: TObject);
var
  AppIni : TMemIniFile;
procedure CreateIniFile;
begin
  AppIni:=TMemIniFile.Create(IniFileFullPath);
  AppIni.WriteString('General','Name','ImageTrimmer');
  AppIni.UpdateFile;
  AppIni.Free;
  MessageDlg('The program created "'+IniFileFullPath+'" file.',mtInformation,[mbOK],0);
end;
begin
  FormMain.Width:=640;
  FormMain.Height:=480;
  FormMain.Caption:=FormMain.Caption+'  '+GetVersionInfo('FileVersion');
  HomeDir:=GetEnvironmentVariable('HOME');
  if HomeDir='' then
  begin
    HomeDir:=ExtractFileDir(ParamStr(0));
  end;
  MainMenuOptionHomeDir.Caption:='Home directory: '+HomeDir;
  MainMenuOptionINIFilename.Caption:='INI filename: '+IniFileName;
  IniFileFullPath:=IncludeTrailingPathDelimiter(HomeDir)+IniFileName;
  if FileExists(IniFileFullPath)=False then
  begin
    CreateIniFile;
    FormMain.Position:=poScreenCenter;
  end;
  ReadOptionsFromFile(IniFileFullPath);
  ReadFormStatusFromIniFile(FormMain);
  DisplayTrimmingSize;
  DisplayJPEGQuality;
  TrimmingBoxIsSelected:=False;
  OriginalCursor:=Screen.Cursor;
  OpenedFilename:='';
  EnableControls(False);
end;





procedure TFormMain.ReadOptionsFromFile(OptFileFullPath: string);
var
  AppIni : TMemIniFile;
  I:         Integer;
  FontStyle: Byte;
begin
  AppIni:=TMemIniFile.Create(OptFileFullPath);
  Font.Name:=AppIni.ReadString('Font', 'FontName', 'Times New Roman');
  FontStyle:=AppIni.ReadInteger('Font', 'FontStyle', 0);
  Font.Style:=TFontStyles(FontStyle);
  Font.Size:=AppIni.ReadInteger('Font', 'FontSize', 9);
  Font.Charset:=TFontCharSet(AppIni.ReadInteger('Font', 'CharacterSet', 0));
    for I:= 0 to Screen.FormCount-1 do
      Screen.Forms[I].Dispatch(Msg);
  LastOpenDir:=AppIni.ReadString('LastDir', 'LastOpenDir', 'C:\');
  LastSaveDir:=AppIni.ReadString('LastDir', 'LastSaveDir', 'C:\');
  TrimmingWidth:=AppIni.ReadInteger('TrimmingSize', 'Width', 100);
  TrimmingHeight:=AppIni.ReadInteger('TrimmingSize', 'Height', 100);
  JPEGQuality:=AppIni.ReadInteger('JPEG', 'Quality', 80);
  AppIni.Free;
end;


procedure TFormMain.WriteOptionsToFile(OptFileFullPath: string);
var
  AppIni : TMemIniFile;
begin
  AppIni:=TMemIniFile.Create(OptFileFullPath);
  AppIni.WriteString('Font', 'FontName', Font.Name);
  AppIni.WriteInteger('Font', 'FontStyle', Byte(Font.Style));
  AppIni.WriteInteger('Font', 'FontSize', Byte(Font.Size));
  AppIni.WriteInteger('Font', 'CharacterSet', Byte(Font.Charset));
  AppIni.WriteString('LastDir', 'LastOpenDir', LastOpenDir);
  AppIni.WriteString('LastDir', 'LastSaveDir', LastSaveDir);
  AppIni.WriteInteger('TrimmingSize', 'Width', TrimmingWidth);
  AppIni.WriteInteger('TrimmingSize', 'Height', TrimmingHeight);
  AppIni.WriteInteger('JPEG', 'Quality', JPEGQuality);
  AppIni.UpdateFile;
  AppIni.Free;
end;


procedure TFormMain.ReadFormStatusFromIniFile(Form: TForm);
var
  AppIni : TMemIniFile;
begin
  AppIni:=TMemIniFile.Create(IniFileFullPath);
  Form.Top:=AppIni.ReadInteger(Form.Name,'Top', Form.Top);
  Form.Left:=AppIni.ReadInteger(Form.Name,'Left', Form.Left);
  Form.Width:=AppIni.ReadInteger(Form.Name,'Width', Form.Width);
  Form.Height:=AppIni.ReadInteger(Form.Name,'Height', Form.Height);
  Form.WindowState:=TWindowState(AppIni.ReadInteger(Form.Name,'WindowState', 0));
  AppIni.Free;
end;


procedure TFormMain.WriteFormStatusToIniFile(Form: TForm);
var
  AppIni : TMemIniFile;
begin
  AppIni:=TMemIniFile.Create(IniFileFullPath);
  AppIni.WriteInteger(Form.Name,'WindowState', Integer(Form.Windowstate));
  AppIni.WriteInteger(Form.Name,'Top', Form.Top);
  AppIni.WriteInteger(Form.Name,'Left', Form.Left);
  AppIni.WriteInteger(Form.Name,'Width', Form.Width);
  AppIni.WriteInteger(Form.Name,'Height', Form.Height);
  AppIni.UpdateFile;
  AppIni.Free;
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
  WriteFormStatusToIniFile(FormMain);
  WriteOptionsToFile(IniFileFullPath);
end;

procedure TFormMain.MainMenuEditFontClick(Sender: TObject);
var
  I: Integer;
begin
  FontDialog.Font:=Font;
  if FontDialog.Execute=True then
  begin
    Font := FontDialog.Font;
    for I:= 0 to Screen.FormCount-1 do
      Screen.Forms[I].Dispatch(Msg);
  end;
end;

procedure TFormMain.MainMenuFileExitClick(Sender: TObject);
begin
  Close;
end;


procedure TFormMain.MainMenuFileLoadClick(Sender: TObject);
var
  bm: TBitmap;
begin
  OpenDialog.InitialDir:=LastOpenDir;
  if OpenDialog.Execute=True then
  begin
    bm:=TBitMap.Create;
    bm.PixelFormat:=pf32bit;
    try
      ImageMaster.Picture.LoadFromFile(OpenDialog.FileName);
      bm.Width:=ImageMaster.Picture.Graphic.Width;
      bm.Height:=ImageMaster.Picture.Graphic.Height;
      LastOpenDir:=ExtractFileDir(OpenDialog.FileName);
      OpenedFilename:=OpenDialog.FileName;
      ComboBoxRatio.Text:='100';
      ImageView.Width:=bm.Width;
      ImageView.Height:=bm.Height;
      bm.Canvas.Draw(0,0, ImageMaster.Picture.Graphic);
      ImageView.Picture.Graphic:=bm;
      ImageMaster.Picture:=Nil;
      ImageMaster.Picture.Graphic:=bm;
      TrimmingTop:=0;
      TrimmingLeft:=0;
      ImageRatio:=1;
      MarkerX:=TrimmingWidth div 2;
      MarkerY:=TrimmingHeight div 2;
      DisplayTrimmingBox;
      MainMenuDisplayOriginalImageSize.Caption:='Original Image Size: '+IntToStr(ImageMaster.Picture.Width)+'x'+IntToStr(ImageMaster.Picture.Height);
      EnableControls(True);
    except
      MessageDlg('Could not open file.', mtWarning, [mbOK], 0);
    end;
  end;
end;


procedure TFormMain.ResizeImage(Ratio: Integer);
var
  AdjustedWidth: Integer;
  AdjustedHeight: Integer;
  R: Single;
  VirtualTrimmingWidth: Integer;
  VirtualTrimmingHeight: Integer;
begin
  if ImageMaster.Picture.Graphic=Nil then
  begin
    UnloadedWarning;
    Exit;
  end;
  R:=Ratio*0.01;
  AdjustImageSize(ImageMaster.Picture.Width, ImageMaster.Picture.Height,
                  round(ImageMaster.Picture.Width*R), round(ImageMaster.Picture.Height*R),
                  AdjustedWidth, AdjustedHeight);
  ImageView.Width:=AdjustedWidth;
  ImageView.Height:=AdjustedHeight;
  ImageView.Canvas.StretchDraw(Rect(0,0,AdjustedWidth-1,AdjustedHeight-1),ImageMaster.Picture.Graphic);
  ImageRatio:=AdjustedWidth/ImageMaster.Picture.Width; (* 1 for full size *)
  if MainMenuOptionAutoTracking.Checked=True then
  begin
    if MainMenuOptionTrimmingBoxFixed.Checked=True then
    begin
      VirtualTrimmingWidth:=TrimmingWidth;
      VirtualTrimmingHeight:=TrimmingHeight;
    end
    else begin
      VirtualTrimmingWidth:=Round(TrimmingWidth*R);
      VirtualTrimmingHeight:=Round(TrimmingHeight*R);
    end;
    TrimmingLeft:=Round(MarkerX*R)-(VirtualTrimmingWidth div 2);
    TrimmingTop:=Round(MarkerY*R)-(VirtualTrimmingHeight div 2);
  end;
  DisplayTrimmingBox;
end;


procedure TFormMain.ComboBoxRatioChange(Sender: TObject);
var
  Ratio: Integer;
begin
  Ratio:=StrToInt(ComboBoxRatio.Text);
  if (Ratio>100) or (Ratio<1) then
    Ratio:=100;
  ResizeImage(Ratio);
end;


procedure TFormMain.MainMenuOptionFontClick(Sender: TObject);
var
  I: Integer;
begin
  FontDialog.Font:=Font;
  if FontDialog.Execute=True then
  begin
    Font := FontDialog.Font;
    for I:= 0 to Screen.FormCount-1 do
      Screen.Forms[I].Dispatch(Msg);
  end;
end;

procedure TFormMain.MainMenuEditSetTrimmingSizeClick(Sender: TObject);
var
  ExitLoop: Boolean;
  NewTrimmingWidth: Integer;
  NewTrimmingHeight: Integer;
begin
  ExitLoop:=False;
  NewTrimmingWidth:=TrimmingWidth;
  NewTrimmingHeight:=TrimmingHeight;
  while ExitLoop=False do
  begin
    try
      NewTrimmingWidth:=StrToInt(InputBox('Trimming width', 'Enter trimming width', IntToStr(TrimmingWidth)));
      if NewTrimmingWidth>0 then
        ExitLoop:=True;
    except
      MessageDlg('Value error.  Please enter again.', mtWarning, [mbOK], 0);
    end;
  end;
  ExitLoop:=False;
  while ExitLoop=False do
  begin
    try
      NewTrimmingHeight:=StrToInt(InputBox('Trimming height', 'Enter trimming height', IntToStr(TrimmingHeight)));
      if NewTrimmingHeight>0 then
        ExitLoop:=True;
    except
      MessageDlg('Value error.  Please enter again.', mtWarning, [mbOK], 0);
    end;
  end;
  DisplayTrimmingBox;
  TrimmingWidth:=NewTrimmingWidth;
  TrimmingHeight:=NewTrimmingHeight;
  DisplayTrimmingBox;
  DisplayTrimmingSize;
end;


procedure TFormMain.DisplayJPEGQuality;
begin
  MainMenuEditJPEGQuality.Caption:='JPEG Quality: '+IntToStr(JPEGQuality);
end;



procedure TFormMain.DisplayTrimmingSize;
begin
  MainMenuEditTrimmingSize.Caption:='Trimming Size: '+IntToStr(TrimmingWidth)+' x '+IntToStr(TrimmingHeight);
end;



procedure TFormMain.DisplayTrimmingBox;
var
  VirtualTrimmingWidth: Integer;
  VirtualTrimmingHeight: Integer;
begin
  if MainMenuOptionTrimmingBoxFixed.Checked=True then
  begin
    VirtualTrimmingWidth:=TrimmingWidth;
    VirtualTrimmingHeight:=TrimmingHeight;
  end
  else begin
    VirtualTrimmingWidth:=Round(TrimmingWidth*ImageRatio);
    VirtualTrimmingHeight:=Round(TrimmingHeight*ImageRatio);
  end;
  ImageView.Canvas.Pen.Mode:=pmNot;
  Imageview.Canvas.MoveTo(TrimmingLeft, TrimmingTop);
  Imageview.Canvas.LineTo(TrimmingLeft+VirtualTrimmingWidth-1, TrimmingTop);
  Imageview.Canvas.LineTo(TrimmingLeft+VirtualTrimmingWidth-1, TrimmingTop+VirtualTrimmingHeight-1);
  Imageview.Canvas.LineTo(TrimmingLeft, TrimmingTop+VirtualTrimmingHeight-1);
  Imageview.Canvas.LineTo(TrimmingLeft, TrimmingTop);
end;






procedure TFormMain.PaintBoxMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  VirtualTrimmingWidth: Integer;
  VirtualTrimmingHeight: Integer;
begin
  if OpenedFileName='' then
    Exit;
  if Button=mbLeft then
  begin
    if (X>=TrimmingLeft)and(X<=(TrimmingLeft+TrimmingWidth))
      and(Y>=TrimmingTop)and(Y<=(TrimmingTop+TrimmingHeight)) then
    begin
      MouseX:=X;
      MouseY:=Y;
      TrimmingBoxIsSelected:=True;
      Screen.Cursor:=crDrag;
    end;
  end;
  if Button=mbRight then
  begin
    if (X>=0)and(X<=ImageView.Width)and(Y>=0)and(Y<=ImageView.Height) then
    begin
      DisplayTrimmingBox;

      (* Set marker position to mouse clicked position *)
      MarkerX:=Round(X/ImageRatio);
      MarkerY:=Round(Y/ImageRatio);

      if MainMenuOptionTrimmingBoxFixed.Checked=True then
      begin
        VirtualTrimmingWidth:=TrimmingWidth;
        VirtualTrimmingHeight:=TrimmingHeight;
      end
      else begin
        VirtualTrimmingWidth:=Round(TrimmingWidth*ImageRatio);
        VirtualTrimmingHeight:=Round(TrimmingHeight*ImageRatio);
      end;
      TrimmingLeft:=Round(MarkerX*ImageRatio)-(VirtualTrimmingWidth div 2);
      TrimmingTop:=Round(MarkerY*ImageRatio)-(VirtualTrimmingHeight div 2);
      DisplayTrimmingBox;
    end;
  end;
end;



procedure TFormMain.PaintBoxMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button=mbLeft then
  begin
    if TrimmingBoxIsSelected then
    begin
      TrimmingBoxIsSelected:=False;
      Screen.Cursor:=OriginalCursor;
    end;
  end;
end;



procedure TFormMain.PaintBoxMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
var
  NewTrimmingLeft: Integer;
  NewTrimmingTop:  Integer;
  NewMouseX:       Integer;
  NewMouseY:       Integer;
  Q:               TPoint;
  VirtualTrimmingWidth: Integer;
  VirtualTrimmingHeight: Integer;
begin
  if MainMenuOptionTrimmingBoxFixed.Checked=True then
  begin
    VirtualTrimmingWidth:=TrimmingWidth;
    VirtualTrimmingHeight:=TrimmingHeight;
  end
  else begin
    VirtualTrimmingWidth:=Round(TrimmingWidth*ImageRatio);
    VirtualTrimmingHeight:=Round(TrimmingHeight*ImageRatio);
  end;

  if TrimmingBoxIsSelected=True then
  begin
    NewMouseX:=X;
    NewMouseY:=Y;

    (* constraint mouse pointer position
     *)
    if (NewMouseX<0)or(NewMouseX>(ImageView.Width-1))
      or (NewMouseY<0)or(NewMouseY>(ImageView.Height-1)) then
    begin
      if NewMouseX<0 then
        NewMouseX:=0;
      if NewMouseX>(ImageView.Width-1) then
        NewMouseX:=ImageView.Width-1;
      if NewMouseY<0 then
        NewMouseY:=0;
      if NewMouseY>(ImageView.Height-1) then
        NewMouseY:=ImageView.Height-1;
      Q:=Point(NewMouseX, NewMouseY);
      Q:=ImageView.ClientToScreen(Q);
      SetCursorPos(Q.X, Q.Y);
    end;

    NewTrimmingLeft:=TrimmingLeft+(NewMouseX-MouseX);
    NewTrimmingTop:=TrimmingTop+(NewMouseY-MouseY);
    if NewTrimmingLeft<0 then
    begin
      NewTrimmingLeft:=0;
      NewMouseX:=MouseX;
    end;
    if NewTrimmingTop<0 then
    begin
      NewTrimmingTop:=0;
      NewMouseY:=MouseY;
    end;
    if (NewTrimmingLeft+VirtualTrimmingWidth)>(ImageView.Width-1) then
    begin
      NewTrimmingLeft:=ImageView.Width-VirtualTrimmingWidth;
      NewMouseX:=MouseX;
    end;
    if (NewTrimmingTop+VirtualTrimmingHeight)>(ImageView.Height-1) then
    begin
      NewTrimmingTop:=ImageView.Height-VirtualTrimmingHeight;
      NewMouseY:=MouseY;
    end;
    DisplayTrimmingBox; (* Erase trimming box *)
    TrimmingLeft:=NewTrimmingLeft;
    TrimmingTop:=NewTrimmingTop;
    DisplayTrimmingBox; (* Re-display trimming box *)
    MarkerX:=Round((TrimmingLeft+(VirtualTrimmingWidth div 2))/ImageRatio);
    MarkerY:=Round((TrimmingTop+(VirtualTrimmingHeight div 2))/ImageRatio);
    MouseX:=NewMouseX;
    MouseY:=NewMouseY;
  end;
end;



procedure TFormMain.MainMenuDisplayTrimminbBoxAtHomePotisionClick(
  Sender: TObject);
begin
  DisplayTrimmingBox;
  TrimmingLeft:=0;
  TrimmingTop:=0;
  DisplayTrimmingBox;
end;

procedure TFormMain.MainMenuFileSaveAsClick(Sender: TObject);
var
  ActualTrimmingLeft: Integer;
  ActualTrimmingTop: Integer;
  JPEG: TJPEGImage;
  RectView: TRect;
  RectSave: TRect;
begin
  if OpenedFilename='' then
  begin
    UnloadedWarning;
    Exit;
  end;

  ImageSave.Width:=TrimmingWidth;
  ImageSave.Height:=TrimmingHeight;

  if MainMenuOptionTrimmingBoxFixed.Checked=True then
  begin
    (* Display warning message and exit procedure if the trimming box
     * is located out of the ImageView.
     *)
    if not((TrimmingLeft>=0)and(TrimmingTop>=0)
            and((TrimmingLeft+TrimmingWidth)<=ImageView.Width)
            and((TrimmingTop+TrimmingHeight)<=ImageView.Height)) then
    begin
      MessageDlg('The trimming box is located out of image.', mtWarning, [mbOK], 0);
      Exit;
    end;
    RectView:=Rect(TrimmingLeft, TrimmingTop, (TrimmingLeft+TrimmingWidth), (TrimmingTop+TrimmingHeight));
    RectSave:=Rect(0, 0, TrimmingWidth, TrimmingHeight);
    DisplayTrimmingBox; (* Erase TrimminbBox *)
    ImageSave.Canvas.CopyRect(RectSave, ImageView.Canvas, RectView);
    DisplayTrimmingBox; (* Re-display TrimmingBox *)
  end
  else if MainMenuOptionTrimmingBoxAutoResize.Checked=True then
  begin
    ActualTrimmingLeft:=Round(TrimmingLeft/ImageRatio);
    ActualTrimmingTop:=Round(TrimmingTop/ImageRatio);
    (* Display warning message if the ACTUAL trimming box is located out of the
     * ImageMaster, if the TrimmingBoxAutoResize is True.
     *)
    if not((TrimmingLeft>=0)and(TrimmingTop>=0)
            and((ActualTrimmingLeft+TrimmingWidth)<=ImageMaster.Picture.Width)
            and((ActualTrimmingTop+TrimmingHeight)<=ImageMaster.Picture.Height)) then
    begin
      MessageDlg('The trimming box is located out of viewing image, or the trimming box is located out original image because of the calculation error.', mtWarning, [mbOK], 0);
      Exit;
    end;
    ActualTrimmingLeft:=Round(TrimmingLeft/ImageRatio);
    ActualTrimmingTop:=Round(TrimmingTop/ImageRatio);
    RectView:=Rect(ActualTrimmingLeft, ActualTrimmingTop, (ActualTrimmingLeft+TrimmingWidth), (ActualTrimmingTop+TrimmingHeight));
    RectSave:=Rect(0, 0, TrimmingWidth, TrimmingHeight);
    ImageSave.Canvas.CopyRect(RectSave, ImageMaster.Canvas, RectView);
  end;

  SaveDialog.InitialDir:=LastSaveDir;
  if SaveDialog.Execute=False then
  begin
    Exit;
  end;

  if SaveDialog.FileName=OpenedFileName then
  begin
    MessageDlg('You can not overwrite original image file.', mtWarning, [mbOK],0);
    Exit;
  end;
  if FileExists(SaveDialog.FileName) then
  begin
    if MessageDlg('The file is already exsited.  The program overwrite the file.  Are you sure?', mtConfirmation, [mbOK, mbCancel],0)=mrCancel then
    begin
      Exit;
    end;
  end;

  JPEG:=TJPEGImage.Create;
  JPEG.Assign(ImageSave.Picture.Bitmap);
  JPEG.CompressionQuality:=JPEGQuality;
  JPEG.SaveToFile(SaveDialog.FileName);
  JPEG.Free;
  LastSaveDir:=ExtractFileDir(SaveDialog.FileName);
end;

procedure TFormMain.MainMenuImageSizeEnlargeClick(Sender: TObject);
var
  Ratio: Integer;
begin
  Ratio:=StrToInt(ComboBoxRatio.Text);
  if (Ratio<100) then
  begin
    inc(Ratio);
    ComboBoxRatio.Text:=IntToStr(Ratio);
    ResizeImage(Ratio);
  end;
end;

procedure TFormMain.MainMenuImageSizeShrinkClick(Sender: TObject);
var
  Ratio: Integer;
begin
  Ratio:=StrToInt(ComboBoxRatio.Text);
  if (Ratio>1) then
  begin
    dec(Ratio);
    ComboBoxRatio.Text:=IntToStr(Ratio);
    ResizeImage(Ratio);
  end;
end;

procedure TFormMain.MainMenuImageSize100Click(Sender: TObject);
begin
  ResizeImage(100);
  ComboBoxRatio.Text:='100';
end;

procedure TFormMain.MainMenuImageSize90Click(Sender: TObject);
begin
  ResizeImage(90);
  ComboBoxRatio.Text:='90';
end;

procedure TFormMain.MainMenuImageSize80Click(Sender: TObject);
begin
  ResizeImage(80);
  ComboBoxRatio.Text:='80';
end;

procedure TFormMain.MainMenuImageSize70Click(Sender: TObject);
begin
  ResizeImage(70);
  ComboBoxRatio.Text:='70';
end;

procedure TFormMain.MainMenuImageSize60Click(Sender: TObject);
begin
  ResizeImage(60);
  ComboBoxRatio.Text:='60';
end;

procedure TFormMain.MainMenuImageSize50Click(Sender: TObject);
begin
  ResizeImage(50);
  ComboBoxRatio.Text:='50';
end;

procedure TFormMain.MainMenuImageSize40Click(Sender: TObject);
begin
  ResizeImage(40);
  ComboBoxRatio.Text:='40';
end;

procedure TFormMain.MainMenuImageSize30Click(Sender: TObject);
begin
  ResizeImage(30);
  ComboBoxRatio.Text:='30';
end;

procedure TFormMain.MainMenuImageSize20Click(Sender: TObject);
begin
  ResizeImage(20);
  ComboBoxRatio.Text:='20';
end;

procedure TFormMain.MainMenuImageSize10Click(Sender: TObject);
begin
  ResizeImage(10);
  ComboBoxRatio.Text:='10';
end;


procedure TFormMain.MainMenuImageSizeJustifyClick(Sender: TObject);
var
  AdjustedWidth: Integer;
  AdjustedHeight: Integer;
  VirtualTrimmingWidth: Integer;
  VirtualTrimmingHeight: Integer;
begin
  AdjustImageSize(ImageMaster.Picture.Width, ImageMaster.Picture.Height,
                  ScrollBox.Width, ScrollBox.Height,
                  AdjustedWidth, AdjustedHeight);
  ImageView.Width:=AdjustedWidth;
  ImageView.Height:=AdjustedHeight;
  ImageView.Canvas.StretchDraw(Rect(0,0,AdjustedWidth-1,AdjustedHeight-1),ImageMaster.Picture.Graphic);
  ImageRatio:=AdjustedWidth/ImageMaster.Picture.Width;

  if MainMenuOptionAutoTracking.Checked=True then
  begin
    if MainMenuOptionTrimmingBoxFixed.Checked=True then
    begin
      VirtualTrimmingWidth:=TrimmingWidth;
      VirtualTrimmingHeight:=TrimmingHeight;
    end
    else begin
      VirtualTrimmingWidth:=Round(TrimmingWidth*ImageRatio);
      VirtualTrimmingHeight:=Round(TrimmingHeight*ImageRatio);
    end;
    TrimmingLeft:=Round(MarkerX*ImageRatio)-(VirtualTrimmingWidth div 2);
    TrimmingTop:=Round(MarkerY*ImageRatio)-(VirtualTrimmingHeight div 2);
  end;

  DisplayTrimmingBox;
  ComboBoxRatio.Text:=IntToStr(Round(ImageRatio*100));
end;

procedure TFormMain.MainMenuSetJPEGQualityClick(Sender: TObject);
var
  ExitLoop: Boolean;
  NewJPEGQuality: Integer;
begin
  ExitLoop:=False;
  NewJPEGQuality:=JPEGQuality;
  while ExitLoop=False do
  begin
    try
      NewJPEGQuality:=StrToInt(InputBox('JPEG quality', 'Enter JPEG quality in %', IntToStr(JPEGQuality)));
      if (NewJPEGQuality>=1)and(NewJPEGQuality<=100) then
        ExitLoop:=True;
    except
      MessageDlg('Value error.  The JPEQ quality must be 1 to 100.  Please enter again.', mtWarning, [mbOK], 0);
    end;
  end;
  JPEGQuality:=NewJPEGQuality;
  DisplayJPEGQuality;
end;


procedure TFormMain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_ADD) then
  begin
    //ShowMessage('+');
    MainMenuImageSizeEnlargeClick(Sender);
  end;
  if (Key=VK_SUBTRACT) then
  begin
    //ShowMessage('-');
    MainMenuImageSizeShrinkClick(Sender);
  end;
end;

procedure TFormMain.MainMenuOptionTrimmingBoxFixedClick(Sender: TObject);
begin
  DisplayTrimmingBox;
  MainMenuOptionTrimmingBoxFixed.Checked:=True;
  MainMenuOptionTrimmingBoxAutoResize.Checked:=False;
  RadioGroupTrimmingBox.ItemIndex:=0;
  DisplayTrimmingBox;
end;

procedure TFormMain.MainMenuOptionTrimmingBoxAutoResizeClick(
  Sender: TObject);
begin
  DisplayTrimmingBox;
  MainMenuOptionTrimmingBoxFixed.Checked:=False;
  MainMenuOptionTrimmingBoxAutoResize.Checked:=True;
  RadioGroupTrimmingBox.ItemIndex:=1;
  DisplayTrimmingBox;
end;

procedure TFormMain.RadioGroupTrimmingBoxClick(Sender: TObject);
begin
  if ImageMaster.Picture.Graphic=Nil then
  begin
    UnloadedWarning;
    Exit;
  end;
  DisplayTrimmingBox;
  if RadioGroupTrimmingBox.ItemIndex=0 then
  begin
    MainMenuOptionTrimmingBoxFixed.Checked:=True;
    MainMenuOptionTrimmingBoxAutoResize.Checked:=False;
  end
  else begin
    MainMenuOptionTrimmingBoxFixed.Checked:=False;
    MainMenuOptionTrimmingBoxAutoResize.Checked:=True;
  end;
  DisplayTrimmingBox;
end;

procedure TFormMain.ScrollBoxMouseWheelDown(Sender: TObject;
  Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
begin
//  ShowMessage('Scrollbox''s wheel down event is occurred.');
//  Handled:=True;
end;

procedure TFormMain.ScrollBoxMouseWheelUp(Sender: TObject;
  Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
begin
//  ShowMessage('Scrollbox''s wheel up event is occurred.');
//  Handled:=True;
end;

procedure TFormMain.FormMouseWheelDown(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
//  ShowMessage('Form''s wheel down event is occurred.');
//  Handled:=True;
end;

procedure TFormMain.FormMouseWheelUp(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
//  ShowMessage('Form''s wheel up event is occurred.');
//  Handled:=True;
end;

end.
